import argparse
import shutil

import click
from pdm import Project, termui
from pdm.cli.commands.base import BaseCommand
from pdm.cli.options import verbose_option

from pdm_venv.utils import get_all_venvs


class PurgeCommand(BaseCommand):
    """Purge selected/all created Virtualenvs"""

    arguments = [verbose_option]

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        parser.add_argument(
            "-f",
            "--force",
            action="store_true",
            help="Force purging without prompting for confirmation",
        )
        parser.add_argument(
            "-i",
            "--interactive",
            action="store_true",
            help="Interatively purge selected Virtualenvs",
        )

    def handle(self, project: Project, options: argparse.Namespace) -> None:

        if not options.force:
            project.core.ui.echo(
                termui.red("The following Virtualenvs will be purged:")
            )
            for i, venv in enumerate(get_all_venvs(project)):
                project.core.ui.echo(f"{i}. {termui.green(venv[0])}")

        if not options.interactive and (
            options.force or click.confirm(termui.yellow("continue? "))
        ):
            self.del_all_venvs(project)

        if options.interactive:
            selection = click.prompt(
                "Please select",
                type=click.Choice(
                    [str(i) for i in range(len(list(get_all_venvs(project))))]
                    + ["all", "none"]
                ),
                default="none",
                show_choices=False,
            )

            if selection == "all":
                self.del_all_venvs(project)
            elif selection != "none":
                for i, venv in enumerate(get_all_venvs(project)):
                    if i == int(selection):
                        shutil.rmtree(venv[1])
                project.core.ui.echo("Purged successfully!")

    def del_all_venvs(self, project):
        for _, venv in get_all_venvs(project):
            shutil.rmtree(venv)
        project.core.ui.echo("Purged successfully!")



import argparse
import shlex
from pathlib import Path

import shellingham
from pdm import termui
from pdm.cli.commands.base import BaseCommand
from pdm.cli.options import verbose_option
from pdm.project import Project
from pdm.utils import is_venv_python

from pdm_venv.utils import BIN_DIR, iter_venvs


class ActivateCommand(BaseCommand):
    """Activate the virtualenv with the given name"""

    arguments = [verbose_option]

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        parser.add_argument("env", nargs="?", help="The key of the virtualenv")

    def handle(self, project: Project, options: argparse.Namespace) -> None:
        if options.env:
            venv = next(
                (venv for key, venv in iter_venvs(project) if key == options.env), None
            )
            if not venv:
                project.core.ui.echo(
                    termui.yellow(f"No virtualenv with key {options.env} is found"),
                    err=True,
                )
                raise SystemExit(1)
        else:
            # Use what is saved in .pdm.toml
            interpreter = project.python_executable
            if is_venv_python(interpreter):
                venv = Path(interpreter).parent.parent
            else:
                project.core.ui.echo(
                    termui.yellow(
                        f"Can't activate a non-venv Python{interpreter}, "
                        "you can specify one with pdm venv activate <env_name>"
                    )
                )
                raise SystemExit(1)
        project.core.ui.echo(self.get_activate_command(venv))

    def get_activate_command(self, venv: Path) -> str:
        shell, _ = shellingham.detect_shell()
        if shell == "fish":
            command, filename = "source", "activate.fish"
        elif shell == "csh":
            command, filename = "source", "activate.csh"
        elif shell in ["powershell", "pwsh"]:
            command, filename = ".", "Activate.ps1"
        else:
            command, filename = "source", "activate"
        activate_script = venv / BIN_DIR / filename
        if activate_script.exists():
            return f"{command} {shlex.quote(str(activate_script))}"
        # Conda backed virtualenvs don't have activate scripts
        return f"conda activate {shlex.quote(str(venv))}"



"""Configuration for the pytest test suite."""
import functools
import os

import pytest
from click.testing import CliRunner
from pdm.core import Core
from pdm.utils import cd

DUMMY_PYPROJECT = """[project]
requires-python = ">=3.6"
dependencies = []
"""


@pytest.fixture
def isolated(tmp_path, mocker, monkeypatch):
    tmp_path.joinpath("pyproject.toml").write_text(DUMMY_PYPROJECT)
    mocker.patch("pathlib.Path.home", return_value=tmp_path)
    monkeypatch.delenv("VIRTUAL_ENV", raising=False)
    return tmp_path


@pytest.fixture
def project(isolated):
    core = Core()
    core.init_parser()
    core.load_plugins()
    p = core.create_project(isolated)
    p.root.joinpath(".global").mkdir()
    p.GLOBAL_PROJECT = p.root.joinpath(".global/project")
    p.global_config["venv.location"] = str(isolated / "venvs")
    p.global_config["venv.backend"] = os.getenv("VENV_BACKEND", "virtualenv")
    return p


@pytest.fixture
def invoke(isolated, monkeypatch):
    runner = CliRunner(mix_stderr=False)
    core = Core()
    invoker = functools.partial(runner.invoke, core, prog_name="pdm")
    monkeypatch.delenv("PDM_IGNORE_SAVED_PYTHON", raising=False)
    with cd(isolated):
        invoker(["config", "venv.location", str(isolated / "venvs")])
        invoker(["config", "venv.backend", os.getenv("VENV_BACKEND", "virtualenv")])
        if (isolated / ".pdm.toml").exists():
            (isolated / ".pdm.toml").unlink()
        yield invoker



import os
import re

import pytest


def test_venv_create(invoke):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    assert os.path.exists(venv_path)


def test_venv_list(invoke):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)

    result = invoke(["venv", "list"])
    assert result.exit_code == 0, result.stderr
    assert venv_path in result.output


def test_venv_remove(invoke):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    key = venv_path.rsplit("-", 1)[-1]

    result = invoke(["venv", "remove", "non-exist"])
    assert result.exit_code != 0

    result = invoke(["venv", "remove", "-y", key])
    assert result.exit_code == 0, result.stderr

    assert not os.path.exists(venv_path)


def test_venv_recreate(invoke):
    invoke(["use", "-f", "python"])
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr

    result = invoke(["venv", "create"])
    assert result.exit_code != 0

    result = invoke(["venv", "create", "-f"])
    assert result.exit_code == 0, result.stderr


def test_venv_activate(invoke, mocker):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    key = venv_path.rsplit("-", 1)[-1]

    mocker.patch("shellingham.detect_shell", return_value=("bash", None))
    result = invoke(["venv", "activate", key])
    assert result.exit_code == 0, result.stderr

    if os.getenv("DEFAULT_BACKEND") == "conda":
        assert result.output.startswith("conda activate")
    else:
        assert result.output.strip("'\"\n").endswith("activate")
        assert result.output.startswith("source")


def test_venv_auto_create(invoke, mocker):
    creator = mocker.patch("pdm_venv.backends.Backend.create")
    invoke(["install"])
    creator.assert_called_once()


def test_venv_purge(invoke, mocker):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    result = invoke(["venv", "purge"], input="y")
    assert result.exit_code == 0, result.stderr
    assert not os.path.exists(venv_path)


def test_venv_purge_force(invoke, mocker):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    result = invoke(["venv", "purge", "-f"])
    assert result.exit_code == 0, result.stderr
    assert not os.path.exists(venv_path)


user_options = [("none", True), ("0", False), ("all", False)]


@pytest.mark.parametrize("user_choices, is_path_exists", user_options)
def test_venv_purge_interactive(invoke, mocker, user_choices, is_path_exists):
    result = invoke(["venv", "create"])
    assert result.exit_code == 0, result.stderr
    venv_path = re.match(
        r"Virtualenv (.+) is created successfully", result.output
    ).group(1)
    result = invoke(["venv", "purge", "-i"], input=user_choices)
    assert result.exit_code == 0, result.stderr
    assert os.path.exists(venv_path) == is_path_exists



from pathlib import Path

from pdm_venv import utils


def test_iter_project_venvs(project):
    venv_parent = Path(project.config["venv.location"])
    venv_prefix = utils.get_venv_prefix(project)
    for name in ("foo", "bar", "baz"):
        venv_parent.joinpath(venv_prefix + name).mkdir(parents=True)
    dot_venv_python = utils.get_venv_python(project.root / ".venv")
    dot_venv_python.parent.mkdir(parents=True)
    dot_venv_python.touch()
    venv_keys = [key for key, _ in utils.iter_venvs(project)]
    assert sorted(venv_keys) == ["bar", "baz", "foo", "in-project"]



import os

import platformdirs
from pdm.project import ConfigItem
from pdm.project.config import ensure_boolean

venv_configs = {
    "venv.location": ConfigItem(
        "Parent directory for virtualenvs",
        os.path.join(platformdirs.user_data_dir("pdm"), "venvs"),
        global_only=True,
    ),
    "venv.backend": ConfigItem(
        "Default backend to create virtualenv", default="virtualenv"
    ),
    "venv.in_project": ConfigItem(
        "Create virtualenv in `.venv` under project root",
        default=False,
        env_var="PDM_VENV_IN_PROJECT",
        coerce=ensure_boolean,
    ),
    # Override the default use_venv value to True
    "use_venv": ConfigItem(
        "Install packages into the activated venv site packages instead of PEP 582",
        True,
        env_var="PDM_USE_VENV",
        coerce=ensure_boolean,
    ),
}



import os
from pathlib import Path
from typing import Iterable, Optional

from pdm import Project as PdmProject
from pdm import termui
from pdm.core import Core
from pdm.models.environment import Environment, GlobalEnvironment
from pdm.models.python import PythonInfo
from pdm.models.specifiers import PySpecSet
from pdm.utils import is_venv_python

from pdm_venv.backends import BACKENDS
from pdm_venv.commands import VenvCommand
from pdm_venv.config import venv_configs
from pdm_venv.utils import get_venv_python, iter_venvs


class Project(PdmProject):
    def find_interpreters(
        self, python_spec: Optional[str] = None
    ) -> Iterable[PythonInfo]:

        for _, venv in iter_venvs(self):
            python = get_venv_python(venv).as_posix()
            py_version = PythonInfo.from_path(python)
            if not python_spec:
                yield py_version
            elif all(d.isdigit() for d in python_spec.split(".")):
                desired = tuple(int(d) for d in python_spec.split("."))
                if py_version.version_tuple[: len(desired)] == desired:
                    yield py_version

        yield from super().find_interpreters(python_spec)

    def get_environment(self) -> Environment:
        if self.is_global:
            env = GlobalEnvironment(self)
            # Rewrite global project's python requires to be
            # compatible with the exact version
            env.python_requires = PySpecSet(f"=={self.python.version}")
            return env
        if self.config["use_venv"]:
            if self.project_config.get("python.path") and not os.getenv(
                "PDM_IGNORE_SAVED_PYTHON"
            ):
                return (
                    GlobalEnvironment(self)
                    if is_venv_python(self.python.executable)
                    else Environment(self)
                )
            if os.getenv("VIRTUAL_ENV"):
                venv = os.getenv("VIRTUAL_ENV")
                self.core.ui.echo(
                    f"Detected inside an active virtualenv {termui.green(venv)}, "
                    "reuse it."
                )
                # Temporary usage, do not save in .pdm.toml
                self._python = PythonInfo.from_path(get_venv_python(Path(venv)))
                return GlobalEnvironment(self)
            existing_venv = next((venv for _, venv in iter_venvs(self)), None)
            if existing_venv:
                self.core.ui.echo(
                    f"Virtualenv {termui.green(str(existing_venv))} is reused.",
                    err=True,
                )
                path = existing_venv
            else:
                # Create a virtualenv using the selected Python interpreter
                self.core.ui.echo(
                    "use_venv is on, creating a virtualenv for this project...",
                    fg="yellow",
                    err=True,
                )
                backend: str = self.config["venv.backend"]
                venv_backend = BACKENDS[backend](self, None)
                path = venv_backend.create(
                    None, (), False, self.config["venv.in_project"]
                )
                self.core.ui.echo(f"Virtualenv {path} is created successfully")
            self.python = PythonInfo.from_path(get_venv_python(path))
            return GlobalEnvironment(self)
        else:
            return Environment(self)


def entry_point(core: Core) -> None:
    core.register_command(VenvCommand)
    for name, item in venv_configs.items():
        core.add_config(name, item)
    core.project_class = Project



import abc
import shutil
import subprocess
import sys
from pathlib import Path
from typing import List, Mapping, Optional, Tuple, Type

from pdm import Project, termui
from pdm.exceptions import PdmUsageError, ProjectError
from pdm.models.python import PythonInfo
from pdm.utils import cached_property

from pdm_venv.utils import get_venv_prefix


class VirtualenvCreateError(ProjectError):
    pass


class Backend(abc.ABC):
    """The base class for virtualenv backends"""

    def __init__(self, project: Project, python: Optional[str]) -> None:
        self.project = project
        self.python = python

    @cached_property
    def _resolved_interpreter(self) -> PythonInfo:
        if not self.python:
            return self.project.python
        try:
            return next(self.project.find_interpreters(self.python))
        except StopIteration:
            raise VirtualenvCreateError(f"Can't find python interpreter {self.python}")

    @property
    def ident(self) -> str:
        """Get the identifier of this virtualenv.
        self.python can be one of:
            3.8
            /usr/bin/python
            3.9.0a4
            python3.8
        """
        return self._resolved_interpreter.identifier

    def subprocess_call(self, cmd: List[str], **kwargs) -> None:
        self.project.core.ui.echo(f"Run command: {cmd}", verbosity=termui.DETAIL)
        try:
            subprocess.check_call(cmd)
        except subprocess.CalledProcessError as e:
            raise VirtualenvCreateError(e) from None

    def _ensure_clean(self, location: Path, force: bool = False) -> None:
        if not location.exists():
            return
        if not force:
            raise VirtualenvCreateError(f"The location {location} is not empty")
        self.project.core.ui.echo(
            f"Cleaning existing target directory {location}", err=True
        )
        shutil.rmtree(location)

    def get_location(self, name: Optional[str]) -> Path:
        venv_parent = Path(self.project.config["venv.location"])
        if not venv_parent.is_dir():
            venv_parent.mkdir(exist_ok=True, parents=True)
        return venv_parent / f"{get_venv_prefix(self.project)}{name or self.ident}"

    def create(
        self,
        name: Optional[str] = None,
        args: Tuple[str] = (),
        force: bool = False,
        in_project: bool = False,
    ) -> Path:
        if in_project:
            location = self.project.root / ".venv"
        else:
            location = self.get_location(name)
        self._ensure_clean(location, force)
        self.perform_create(location, args)
        return location

    @abc.abstractmethod
    def perform_create(self, location: Path, args: Tuple[str] = ()) -> Path:
        pass


class VirtualenvBackend(Backend):
    def perform_create(self, location: Path, args: Tuple[str] = ()) -> Path:
        cmd = [sys.executable, "-m", "virtualenv", str(location)]
        cmd.extend(["-p", self._resolved_interpreter.executable])
        cmd.extend(args)
        self.subprocess_call(cmd)


class VenvBackend(VirtualenvBackend):
    def perform_create(self, location: Path, args: Tuple[str]) -> Path:
        cmd = [
            self._resolved_interpreter.executable,
            "-m",
            "venv",
            str(location),
        ] + list(args)
        self.subprocess_call(cmd)


class CondaBackend(Backend):
    @property
    def ident(self) -> str:
        # Conda supports specifying python that doesn't exist,
        # use the passed-in name directly
        if self.python:
            return self.python
        return super().ident

    def perform_create(self, location: Path, args: Tuple[str]) -> Path:
        if self.python:
            python_ver = self.python
        else:
            python = self._resolved_interpreter
            python_ver = f"{python.major}.{python.minor}"
        if any(arg.startswith("python=") for arg in args):
            raise PdmUsageError("Cannot use python= in conda creation arguments")
        cmd = [
            "conda",
            "create",
            "--yes",
            "--prefix",
            str(location),
            # Ensure the pip package is installed.
            "pip",
            f"python={python_ver}",
            *args,
        ]

        self.subprocess_call(cmd)


BACKENDS: Mapping[str, Type[Backend]] = {
    "virtualenv": VirtualenvBackend,
    "venv": VenvBackend,
    "conda": CondaBackend,
}



import argparse

from pdm import Project
from pdm.cli.commands.base import BaseCommand

from pdm_venv.commands.activate import ActivateCommand
from pdm_venv.commands.create import CreateCommand
from pdm_venv.commands.list import ListCommand
from pdm_venv.commands.purge import PurgeCommand
from pdm_venv.commands.remove import RemoveCommand


class VenvCommand(BaseCommand):
    """Virtualenv management"""

    name = "venv"
    arguments = []

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        subparser = parser.add_subparsers()
        CreateCommand.register_to(subparser, "create")
        ListCommand.register_to(subparser, "list")
        RemoveCommand.register_to(subparser, "remove")
        ActivateCommand.register_to(subparser, "activate")
        PurgeCommand.register_to(subparser, "purge")
        self.parser = parser

    def handle(self, project: Project, options: argparse.Namespace) -> None:
        self.parser.print_help()



import base64
import hashlib
import sys
from pathlib import Path
from typing import Iterable, Tuple

from pdm.project import Project
from pdm.utils import get_in_project_venv_python

IS_WIN = sys.platform == "win32"
BIN_DIR = "Scripts" if IS_WIN else "bin"


def hash_path(path: str) -> str:
    """Generate a hash for the given path."""
    return base64.urlsafe_b64encode(hashlib.md5(path.encode()).digest()).decode()[:8]


def get_venv_prefix(project: Project) -> str:
    """Get the venv prefix for the project"""
    path = project.root
    name_hash = hash_path(path.as_posix())
    return f"{path.name}-{name_hash}-"


def iter_venvs(project: Project) -> Iterable[Tuple[str, Path]]:
    """Return an iterable of venv paths associated with the project"""
    in_project_venv_python = get_in_project_venv_python(project.root)
    if in_project_venv_python is not None:
        yield "in-project", Path(in_project_venv_python).parent.parent
    venv_prefix = get_venv_prefix(project)
    venv_parent = Path(project.config["venv.location"])
    for venv in venv_parent.glob(f"{venv_prefix}*"):
        ident = venv.name[len(venv_prefix) :]
        yield ident, venv


def get_venv_python(venv: Path) -> Path:
    """Get the interpreter path inside the given venv."""
    suffix = ".exe" if IS_WIN else ""
    return venv / BIN_DIR / f"python{suffix}"


def get_all_venvs(project: Project) -> Iterable[Tuple[str, Path]]:
    """Return a list of all created venv paths"""
    venv_parent = Path(project.config["venv.location"])
    for venv in venv_parent.glob("*"):
        ident = venv.name
        yield ident, venv



import sys

from pdm_venv.backends import CondaBackend, VenvBackend, VirtualenvBackend


def test_virtualenv_backend_create(project, mocker):
    interpreter = project.python_executable
    backend = VirtualenvBackend(project, None)
    assert backend.ident
    mock_call = mocker.patch("subprocess.check_call")
    location = backend.create()
    mock_call.assert_called_once_with(
        [sys.executable, "-m", "virtualenv", str(location), "-p", interpreter]
    )


def test_venv_backend_create(project, mocker):
    interpreter = project.python_executable
    backend = VenvBackend(project, None)
    assert backend.ident
    mock_call = mocker.patch("subprocess.check_call")
    location = backend.create()
    mock_call.assert_called_once_with([interpreter, "-m", "venv", str(location)])


def test_conda_backend_create(project, mocker):
    backend = CondaBackend(project, "3.8")
    assert backend.ident == "3.8"
    mock_call = mocker.patch("subprocess.check_call")
    location = backend.create()
    mock_call.assert_called_once_with(
        ["conda", "create", "--yes", "--prefix", str(location), "pip", "python=3.8"]
    )

    backend = CondaBackend(project, None)
    python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
    assert backend.ident == python_version
    location = backend.create()
    mock_call.assert_called_with(
        [
            "conda",
            "create",
            "--yes",
            "--prefix",
            str(location),
            "pip",
            f"python={python_version}",
        ]
    )



import argparse

from pdm import BaseCommand, Project, termui
from pdm.cli.options import verbose_option

from pdm_venv.backends import BACKENDS


class CreateCommand(BaseCommand):
    """Create a virtualenv

    pdm venv create <python> [-other args]
    """

    description = "Create a virtualenv"
    arguments = [verbose_option]

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        parser.add_argument(
            "-w",
            "--with",
            dest="backend",
            choices=BACKENDS.keys(),
            help="Specify the backend to create the virtualenv",
        )
        parser.add_argument(
            "-f",
            "--force",
            action="store_true",
            help="Recreate if the virtualenv already exists",
        )
        parser.add_argument("-n", "--name", help="Specify the name of the virtualenv")
        parser.add_argument(
            "python",
            nargs="?",
            help="Specify which python should be used to create the virtualenv",
        )
        parser.add_argument(
            "venv_args",
            nargs=argparse.REMAINDER,
            help="Additional arguments that will be passed to the backend",
        )

    def handle(self, project: Project, options: argparse.Namespace) -> None:
        in_project = (
            project.config["venv.in_project"] if not options.name else False
        )  # ignore venv.in_project flag if name is given
        backend: str = options.backend or project.config["venv.backend"]
        venv_backend = BACKENDS[backend](project, options.python)
        with project.core.ui.open_spinner(
            f"Creating virtualenv using {backend}..."
        ) as spinner:
            path = venv_backend.create(
                options.name, options.venv_args, options.force, in_project
            )
            spinner.succeed(
                f"Virtualenv {termui.green(str(path))} is created successfully"
            )



"""
    pdm-venv

    A plugin for pdm that enables virtualenv management
    :author: Frost Ming <mianghong@gmail.com>
    :license: MIT
"""

from typing import List

__all__: List[str] = []  # noqa: WPS410 (the only __variable__ we use)



from pathlib import Path


def test_use_venv_default_to_true(invoke):
    result = invoke(["config", "use_venv"])
    assert result.output.strip() == "True"


def test_create_venv_first_time(invoke, project):
    result = invoke(["install"])
    assert result.exit_code == 0
    venv_parent = project.root / "venvs"
    venv_path = next(venv_parent.iterdir(), None)
    assert venv_path is not None

    assert Path(project.project_config["python.path"]).relative_to(venv_path)


def test_create_venv_in_project(invoke, project):
    project.project_config["venv.in_project"] = True
    result = invoke(["install"])
    assert result.exit_code == 0
    assert project.root.joinpath(".venv").exists()


def test_find_interpreters_from_venv(invoke, project):
    result = invoke(["install"])
    assert result.exit_code == 0
    venv_parent = project.root / "venvs"
    venv_path = next(venv_parent.iterdir(), None)

    assert Path(next(project.find_interpreters()).executable).relative_to(venv_path)



import argparse
import shutil
from pathlib import Path

import click
from pdm import Project, termui
from pdm.cli.commands.base import BaseCommand
from pdm.cli.options import verbose_option

from pdm_venv.utils import iter_venvs


class RemoveCommand(BaseCommand):
    """Remove the virtualenv with the given name"""

    arguments = [verbose_option]

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        parser.add_argument(
            "-y",
            "--yes",
            action="store_true",
            help="Answer yes on the following question",
        )
        parser.add_argument("env", help="The key of the virtualenv")

    def handle(self, project: Project, options: argparse.Namespace) -> None:
        project.core.ui.echo("Virtualenvs created with this project:")
        for ident, venv in iter_venvs(project):
            if ident == options.env:
                if options.yes or click.confirm(
                    termui.yellow(f"Will remove: {venv}, continue?")
                ):
                    shutil.rmtree(venv)
                    if (
                        project.project_config.get("python.path")
                        and Path(project.project_config["python.path"]).parent.parent
                        == venv
                    ):
                        del project.project_config["python.path"]
                    project.core.ui.echo("Removed successfully!")
                break
        else:
            project.core.ui.echo(
                termui.yellow(f"No virtualenv with key {options.env} is found"),
                err=True,
            )
            raise SystemExit(1)
